from .kd_mix_wrapper import KDMixWrapper
from .label_smoothing_wrapper import LabelSmoothingWrapper
from .multi_view_wrapper import MultiViewWrapper
from .one_hot_wrapper import OneHotWrapper
from .x_transform_wrapper import XTransformWrapper
